"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var classNames = require("classnames");
var d3_array_1 = require("d3-array");
var d3_scale_1 = require("d3-scale");
var d3_shape_1 = require("d3-shape");
var lodash_1 = require("lodash");
var React = require("react");
var types_1 = require("../../helpers/types");
var theme_1 = require("../theme");
require("./AdvancedTimeline.css");
require("./LineChart.css");
var LEGEND_LINE_HEIGHT = 16;
var AdvancedTimeline = (function (_super) {
    __extends(AdvancedTimeline, _super);
    function AdvancedTimeline(props) {
        var _this = _super.call(this, props) || this;
        _this.getRatingScale = function (availableHeight) {
            return d3_scale_1.scalePoint().domain([5, 4, 3, 2, 1]).range([availableHeight, 0]);
        };
        _this.getLevelScale = function (availableHeight) {
            return d3_scale_1.scalePoint().domain(['ERROR', 'WARN', 'OK']).range([availableHeight, 0]);
        };
        _this.getYScale = function (props, availableHeight, flatData) {
            if (props.metricType === 'RATING') {
                return _this.getRatingScale(availableHeight);
            }
            else if (props.metricType === 'LEVEL') {
                return _this.getLevelScale(availableHeight);
            }
            else {
                return d3_scale_1.scaleLinear()
                    .range([availableHeight, 0])
                    .domain([0, d3_array_1.max(flatData, function (d) { return Number(d.y || 0); }) || 1])
                    .nice();
            }
        };
        _this.getXScale = function (_a, availableWidth, flatData) {
            var startDate = _a.startDate, endDate = _a.endDate;
            var dateRange = d3_array_1.extent(flatData, function (d) { return d.x; });
            var start = startDate && startDate > dateRange[0] ? startDate : dateRange[0];
            var end = endDate && endDate < dateRange[1] ? endDate : dateRange[1];
            var xScale = d3_scale_1.scaleTime()
                .domain(lodash_1.sortBy([start, end]))
                .range([0, availableWidth])
                .clamp(false);
            return {
                xScale: xScale,
                maxXRange: dateRange.map(xScale),
            };
        };
        _this.getScales = function (props) {
            var availableWidth = props.width - props.padding[1] - props.padding[3];
            var availableHeight = props.height - props.padding[0] - props.padding[2];
            var flatData = lodash_1.flatten(props.series.map(function (serie) { return serie.data; }));
            return __assign(__assign({}, _this.getXScale(props, availableWidth, flatData)), { yScale: _this.getYScale(props, availableHeight, flatData) });
        };
        _this.getSelectedDatePos = function (xScale, selectedDate) {
            var firstSerie = _this.props.series[0];
            if (selectedDate && firstSerie) {
                var idx = firstSerie.data.findIndex(function (p) { return p.x.valueOf() === selectedDate.valueOf(); });
                var xRange = lodash_1.sortBy(xScale.range());
                var xPos = xScale(selectedDate);
                if (idx >= 0 && xPos >= xRange[0] && xPos <= xRange[1]) {
                    return {
                        selectedDate: selectedDate,
                        selectedDateXPos: xScale(selectedDate),
                        selectedDateIdx: idx,
                    };
                }
            }
            return { selectedDate: undefined, selectedDateXPos: undefined, selectedDateIdx: undefined };
        };
        _this.getEventMarker = function (size) {
            var half = size / 2;
            return "M" + half + " 0 L" + size + " " + half + " L " + half + " " + size + " L0 " + half + " L" + half + " 0 L" + size + " " + half;
        };
        _this.handleWheel = function (event) {
            event.preventDefault();
            var _a = _this.state, maxXRange = _a.maxXRange, xScale = _a.xScale;
            var parentBbox = event.currentTarget.getBoundingClientRect();
            var mouseXPos = (event.pageX - parentBbox.left) / parentBbox.width;
            var xRange = xScale.range();
            var speed = event.deltaMode
                ? (25 / event.deltaMode) * _this.props.zoomSpeed
                : _this.props.zoomSpeed;
            var leftPos = xRange[0] - Math.round(speed * event.deltaY * mouseXPos);
            var rightPos = xRange[1] + Math.round(speed * event.deltaY * (1 - mouseXPos));
            var startDate = leftPos > maxXRange[0] ? xScale.invert(leftPos) : undefined;
            var endDate = rightPos < maxXRange[1] ? xScale.invert(rightPos) : undefined;
            _this.handleZoomUpdate(startDate, endDate);
        };
        _this.handleZoomUpdate = function (startDate, endDate) {
            if (_this.props.updateZoom) {
                _this.props.updateZoom(startDate, endDate);
            }
        };
        _this.handleMouseMove = function (event) {
            var parentBbox = event.currentTarget.getBoundingClientRect();
            _this.updateTooltipPos(event.pageX - parentBbox.left);
        };
        _this.handleMouseEnter = function () {
            _this.setState({ mouseOver: true });
        };
        _this.handleMouseOut = function () {
            var updateTooltip = _this.props.updateTooltip;
            if (updateTooltip) {
                _this.setState({
                    mouseOver: false,
                    selectedDate: undefined,
                    selectedDateXPos: undefined,
                    selectedDateIdx: undefined,
                });
                updateTooltip(undefined, undefined, undefined);
            }
        };
        _this.handleClick = function () {
            var updateSelectedDate = _this.props.updateSelectedDate;
            if (updateSelectedDate) {
                updateSelectedDate(_this.state.selectedDate || undefined);
            }
        };
        _this.setLeakLegendTextWidth = function (node) {
            if (node) {
                _this.setState({ leakLegendTextWidth: node.getBoundingClientRect().width });
            }
        };
        _this.updateTooltipPos = function (xPos) {
            _this.setState(function (state) {
                var firstSerie = _this.props.series[0];
                if (state.mouseOver && firstSerie) {
                    var updateTooltip = _this.props.updateTooltip;
                    var date = state.xScale.invert(xPos);
                    var bisectX = d3_array_1.bisector(function (d) { return d.x; }).right;
                    var idx = bisectX(firstSerie.data, date);
                    if (idx >= 0) {
                        var previousPoint = firstSerie.data[idx - 1];
                        var nextPoint = firstSerie.data[idx];
                        if (!nextPoint ||
                            (previousPoint &&
                                date.valueOf() - previousPoint.x.valueOf() <= nextPoint.x.valueOf() - date.valueOf())) {
                            idx--;
                        }
                        var selectedDate = firstSerie.data[idx].x;
                        var xPos_1 = state.xScale(selectedDate);
                        if (updateTooltip) {
                            updateTooltip(selectedDate, xPos_1, idx);
                        }
                        return { selectedDate: selectedDate, selectedDateXPos: xPos_1, selectedDateIdx: idx };
                    }
                }
                return null;
            });
        };
        _this.renderHorizontalGrid = function () {
            var formatYTick = _this.props.formatYTick;
            var _a = _this.state, xScale = _a.xScale, yScale = _a.yScale;
            var hasTicks = typeof yScale.ticks === 'function';
            var ticks = hasTicks
                ? yScale.ticks(_this.props.maxYTicksCount)
                : yScale.domain();
            if (!ticks.length) {
                ticks.push(yScale.domain()[1]);
            }
            if (formatYTick) {
                var formattedTicks = ticks.map(function (tick) { return formatYTick(tick); });
                if (ticks.length > lodash_1.uniq(formattedTicks).length) {
                    ticks = yScale.domain();
                }
            }
            return (React.createElement("g", null, ticks.map(function (tick) { return (React.createElement("g", { key: tick },
                formatYTick != null && (React.createElement("text", { className: "line-chart-tick line-chart-tick-x", dx: "-1em", dy: "0.3em", textAnchor: "end", x: xScale.range()[0], y: yScale(tick) }, formatYTick(tick))),
                React.createElement("line", { className: "line-chart-grid", x1: xScale.range()[0], x2: xScale.range()[1], y1: yScale(tick), y2: yScale(tick) }))); })));
        };
        _this.renderXAxisTicks = function () {
            var _a = _this.state, xScale = _a.xScale, yScale = _a.yScale;
            var format = xScale.tickFormat(7);
            var ticks = xScale.ticks(7);
            var y = yScale.range()[0];
            return (React.createElement("g", { transform: "translate(0, 20)" }, ticks.slice(0, -1).map(function (tick, index) {
                var nextTick = index + 1 < ticks.length ? ticks[index + 1] : xScale.domain()[1];
                var x = (xScale(tick) + xScale(nextTick)) / 2;
                return (React.createElement("text", { className: "line-chart-tick", key: index, textAnchor: "end", transform: "rotate(-35, " + x + ", " + y + ")", x: x, y: y }, format(tick)));
            })));
        };
        _this.renderNewCodeLegend = function (params) {
            var leakStart = params.leakStart, leakWidth = params.leakWidth, theme = params.theme;
            var _a = _this.state, leakLegendTextWidth = _a.leakLegendTextWidth, xScale = _a.xScale, yScale = _a.yScale;
            var yRange = yScale.range();
            var xRange = xScale.range();
            var legendMinWidth = (leakLegendTextWidth || 0) + theme.rawSizes.grid;
            var legendPadding = theme.rawSizes.grid / 2;
            var legendBackgroundPosition;
            var legendBackgroundWidth;
            var legendMargin;
            var legendPosition;
            var legendTextAnchor;
            if (leakWidth >= legendMinWidth) {
                legendBackgroundWidth = leakWidth;
                legendBackgroundPosition = leakStart;
                legendMargin = 0;
                legendPosition = legendBackgroundPosition + legendPadding;
                legendTextAnchor = 'start';
            }
            else {
                legendBackgroundWidth = legendMinWidth;
                legendBackgroundPosition = xRange[xRange.length - 1] - legendBackgroundWidth;
                legendMargin = theme.rawSizes.grid / 2;
                legendPosition = xRange[xRange.length - 1] - legendPadding;
                legendTextAnchor = 'end';
            }
            return (React.createElement(React.Fragment, null,
                React.createElement("rect", { fill: theme.colors.leakPrimaryColor, height: LEGEND_LINE_HEIGHT, width: legendBackgroundWidth, x: legendBackgroundPosition, y: yRange[yRange.length - 1] - LEGEND_LINE_HEIGHT - legendMargin }),
                React.createElement("text", { className: "new-code-legend", ref: _this.setLeakLegendTextWidth, x: legendPosition, y: yRange[yRange.length - 1] - legendPadding - legendMargin, textAnchor: legendTextAnchor }, "new code")));
        };
        _this.renderLeak = function () {
            var _a = _this.props, displayNewCodeLegend = _a.displayNewCodeLegend, leakPeriodDate = _a.leakPeriodDate;
            if (!leakPeriodDate) {
                return null;
            }
            var _b = _this.state, xScale = _b.xScale, yScale = _b.yScale;
            var yRange = yScale.range();
            var xRange = xScale.range();
            var leakStart = Math.max(xScale(leakPeriodDate), xRange[0]);
            var leakWidth = xRange[xRange.length - 1] - leakStart;
            if (leakWidth < 1) {
                return null;
            }
            return (React.createElement(theme_1.ThemeConsumer, null, function (theme) { return (React.createElement(React.Fragment, null,
                displayNewCodeLegend && _this.renderNewCodeLegend({ leakStart: leakStart, leakWidth: leakWidth, theme: theme }),
                React.createElement("rect", { className: "leak-chart-rect", fill: theme.colors.leakPrimaryColor, height: yRange[0] - yRange[yRange.length - 1], width: leakWidth, x: leakStart, y: yRange[yRange.length - 1] }))); }));
        };
        _this.renderLines = function () {
            var lineGenerator = d3_shape_1.line()
                .defined(function (d) { return Boolean(d.y || d.y === 0); })
                .x(function (d) { return _this.state.xScale(d.x); })
                .y(function (d) { return _this.state.yScale(d.y); });
            if (_this.props.basisCurve) {
                lineGenerator.curve(d3_shape_1.curveBasis);
            }
            return (React.createElement("g", null, _this.props.series.map(function (serie, idx) { return (React.createElement("path", { className: classNames('line-chart-path', 'line-chart-path-' + idx), d: lineGenerator(serie.data) || undefined, key: serie.name })); })));
        };
        _this.renderDots = function () {
            return (React.createElement("g", null, _this.props.series
                .map(function (serie, serieIdx) {
                return serie.data
                    .map(function (point, idx) {
                    var pointNotDefined = !point.y && point.y !== 0;
                    var hasPointBefore = serie.data[idx - 1] && (serie.data[idx - 1].y || serie.data[idx - 1].y === 0);
                    var hasPointAfter = serie.data[idx + 1] && (serie.data[idx + 1].y || serie.data[idx + 1].y === 0);
                    if (pointNotDefined || hasPointBefore || hasPointAfter) {
                        return undefined;
                    }
                    return (React.createElement("circle", { className: classNames('line-chart-dot', 'line-chart-dot-' + serieIdx), cx: _this.state.xScale(point.x), cy: _this.state.yScale(point.y), key: serie.name + idx, r: "2" }));
                })
                    .filter(types_1.isDefined);
            })
                .filter(function (dots) { return dots.length > 0; })));
        };
        _this.renderAreas = function () {
            var areaGenerator = d3_shape_1.area()
                .defined(function (d) { return Boolean(d.y || d.y === 0); })
                .x(function (d) { return _this.state.xScale(d.x); })
                .y1(function (d) { return _this.state.yScale(d.y); })
                .y0(_this.state.yScale(0));
            if (_this.props.basisCurve) {
                areaGenerator.curve(d3_shape_1.curveBasis);
            }
            return (React.createElement("g", null, _this.props.series.map(function (serie, idx) { return (React.createElement("path", { className: classNames('line-chart-area', 'line-chart-area-' + idx), d: areaGenerator(serie.data) || undefined, key: serie.name })); })));
        };
        _this.renderSelectedDate = function () {
            var _a = _this.state, selectedDateIdx = _a.selectedDateIdx, selectedDateXPos = _a.selectedDateXPos, yScale = _a.yScale;
            var firstSerie = _this.props.series[0];
            if (selectedDateIdx == null || selectedDateXPos == null || !firstSerie) {
                return null;
            }
            return (React.createElement("g", null,
                React.createElement("line", { className: "line-tooltip", x1: selectedDateXPos, x2: selectedDateXPos, y1: yScale.range()[0], y2: yScale.range()[1] }),
                _this.props.series.map(function (serie, idx) {
                    var point = serie.data[selectedDateIdx];
                    if (!point || (!point.y && point.y !== 0)) {
                        return null;
                    }
                    return (React.createElement("circle", { className: classNames('line-chart-dot', 'line-chart-dot-' + idx), cx: selectedDateXPos, cy: yScale(point.y), key: serie.name, r: "4" }));
                })));
        };
        _this.renderClipPath = function () {
            return (React.createElement("defs", null,
                React.createElement("clipPath", { id: "chart-clip" },
                    React.createElement("rect", { height: _this.state.yScale.range()[0] + 10, transform: "translate(0,-5)", width: _this.state.xScale.range()[1] }))));
        };
        _this.renderMouseEventsOverlay = function (zoomEnabled) {
            var mouseEvents = {};
            if (zoomEnabled) {
                mouseEvents.onWheel = _this.handleWheel;
            }
            if (_this.props.updateTooltip) {
                mouseEvents.onMouseEnter = _this.handleMouseEnter;
                mouseEvents.onMouseMove = _this.handleMouseMove;
                mouseEvents.onMouseOut = _this.handleMouseOut;
            }
            if (_this.props.updateSelectedDate) {
                mouseEvents.onClick = _this.handleClick;
            }
            return (React.createElement("rect", __assign({ className: "chart-mouse-events-overlay", height: _this.state.yScale.range()[0], width: _this.state.xScale.range()[1] }, mouseEvents)));
        };
        var scales = _this.getScales(props);
        var selectedDatePos = _this.getSelectedDatePos(scales.xScale, props.selectedDate);
        _this.state = __assign(__assign({}, scales), selectedDatePos);
        _this.updateTooltipPos = lodash_1.throttle(_this.updateTooltipPos, 40);
        _this.handleZoomUpdate = lodash_1.throttle(_this.handleZoomUpdate, 40);
        return _this;
    }
    AdvancedTimeline.prototype.componentDidUpdate = function (prevProps) {
        var scales;
        var selectedDatePos;
        if (this.props.metricType !== prevProps.metricType ||
            this.props.startDate !== prevProps.startDate ||
            this.props.endDate !== prevProps.endDate ||
            this.props.width !== prevProps.width ||
            this.props.padding !== prevProps.padding ||
            this.props.height !== prevProps.height ||
            this.props.series !== prevProps.series) {
            scales = this.getScales(this.props);
            if (this.state.selectedDate != null) {
                selectedDatePos = this.getSelectedDatePos(scales.xScale, this.state.selectedDate);
            }
        }
        if (!lodash_1.isEqual(this.props.selectedDate, prevProps.selectedDate)) {
            var xScale = scales ? scales.xScale : this.state.xScale;
            selectedDatePos = this.getSelectedDatePos(xScale, this.props.selectedDate);
        }
        if (scales || selectedDatePos) {
            if (scales) {
                this.setState(__assign({}, scales));
            }
            if (selectedDatePos) {
                this.setState(__assign({}, selectedDatePos));
            }
            if (selectedDatePos && this.props.updateTooltip) {
                this.props.updateTooltip(selectedDatePos.selectedDate, selectedDatePos.selectedDateXPos, selectedDatePos.selectedDateIdx);
            }
        }
    };
    AdvancedTimeline.prototype.render = function () {
        if (!this.props.width || !this.props.height) {
            return React.createElement("div", null);
        }
        var zoomEnabled = !this.props.disableZoom && this.props.updateZoom != null;
        var isZoomed = Boolean(this.props.startDate || this.props.endDate);
        return (React.createElement("svg", { className: classNames('line-chart', { 'chart-zoomed': isZoomed }), height: this.props.height, width: this.props.width },
            zoomEnabled && this.renderClipPath(),
            React.createElement("g", { transform: "translate(" + this.props.padding[3] + ", " + this.props.padding[0] + ")" },
                this.props.leakPeriodDate != null && this.renderLeak(),
                !this.props.hideGrid && this.renderHorizontalGrid(),
                !this.props.hideXAxis && this.renderXAxisTicks(),
                this.props.showAreas && this.renderAreas(),
                this.renderLines(),
                this.renderDots(),
                this.renderSelectedDate(),
                this.renderMouseEventsOverlay(zoomEnabled))));
    };
    AdvancedTimeline.defaultProps = {
        eventSize: 8,
        maxYTicksCount: 4,
        padding: [26, 10, 50, 60],
        zoomSpeed: 1,
    };
    return AdvancedTimeline;
}(React.PureComponent));
exports.default = AdvancedTimeline;
//# sourceMappingURL=AdvancedTimeline.js.map