"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var d3_array_1 = require("d3-array");
var d3_scale_1 = require("d3-scale");
var React = require("react");
var Tooltip_1 = require("../controls/Tooltip");
require("./BarChart.css");
var BarChart = (function (_super) {
    __extends(BarChart, _super);
    function BarChart() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.handleClick = function (point) {
            if (_this.props.onBarClick) {
                _this.props.onBarClick(point);
            }
        };
        _this.renderXTicks = function (xScale, yScale) {
            var _a = _this.props, data = _a.data, _b = _a.xTicks, xTicks = _b === void 0 ? [] : _b;
            if (!xTicks.length) {
                return null;
            }
            var ticks = xTicks.map(function (tick, index) {
                var point = data[index];
                var x = Math.round(xScale(point.x) + xScale.bandwidth() / 2);
                var y = yScale.range()[0];
                var d = data[index];
                var text = (React.createElement("text", { className: "bar-chart-tick", dy: "1.5em", key: index, onClick: function () { return _this.handleClick(point); }, style: { cursor: _this.props.onBarClick ? 'pointer' : 'default' }, x: x, y: y }, tick));
                return (React.createElement(Tooltip_1.default, { key: index, overlay: d.tooltip || undefined }, text));
            });
            return React.createElement("g", null, ticks);
        };
        _this.renderXValues = function (xScale, yScale) {
            var _a = _this.props, data = _a.data, _b = _a.xValues, xValues = _b === void 0 ? [] : _b;
            if (!xValues.length) {
                return null;
            }
            var ticks = xValues.map(function (value, index) {
                var point = data[index];
                var x = Math.round(xScale(point.x) + xScale.bandwidth() / 2);
                var y = yScale(point.y);
                var text = (React.createElement("text", { className: "bar-chart-tick", dy: "-1em", key: index, onClick: function () { return _this.handleClick(point); }, style: { cursor: _this.props.onBarClick ? 'pointer' : 'default' }, x: x, y: y }, value));
                return (React.createElement(Tooltip_1.default, { key: index, overlay: point.tooltip || undefined }, text));
            });
            return React.createElement("g", null, ticks);
        };
        _this.renderBars = function (xScale, yScale) {
            var bars = _this.props.data.map(function (point, index) {
                var x = Math.round(xScale(point.x));
                var maxY = yScale.range()[0];
                var y = Math.round(yScale(point.y)) - 1;
                var height = maxY - y;
                var rect = (React.createElement("rect", { className: "bar-chart-bar", height: height, key: index, onClick: function () { return _this.handleClick(point); }, style: { cursor: _this.props.onBarClick ? 'pointer' : 'default' }, width: _this.props.barsWidth, x: x, y: y }));
                return (React.createElement(Tooltip_1.default, { key: index, overlay: point.tooltip || undefined }, rect));
            });
            return React.createElement("g", null, bars);
        };
        return _this;
    }
    BarChart.prototype.render = function () {
        var _a = this.props, barsWidth = _a.barsWidth, data = _a.data, width = _a.width, height = _a.height, _b = _a.padding, padding = _b === void 0 ? [10, 10, 10, 10] : _b;
        var availableWidth = width - padding[1] - padding[3];
        var availableHeight = height - padding[0] - padding[2];
        var innerPadding = (availableWidth - barsWidth * data.length) / (data.length - 1);
        var relativeInnerPadding = innerPadding / (innerPadding + barsWidth);
        var maxY = d3_array_1.max(data, function (d) { return d.y; });
        var xScale = d3_scale_1.scaleBand()
            .domain(data.map(function (d) { return d.x; }))
            .range([0, availableWidth])
            .paddingInner(relativeInnerPadding);
        var yScale = d3_scale_1.scaleLinear().domain([0, maxY]).range([availableHeight, 0]);
        return (React.createElement("svg", { className: "bar-chart", height: height, width: width },
            React.createElement("g", { transform: "translate(" + padding[3] + ", " + padding[0] + ")" },
                this.renderXTicks(xScale, yScale),
                this.renderXValues(xScale, yScale),
                this.renderBars(xScale, yScale))));
    };
    return BarChart;
}(React.PureComponent));
exports.default = BarChart;
//# sourceMappingURL=BarChart.js.map