import { ScaleLinear } from 'd3-scale';
import * as React from 'react';
import { Link } from 'react-router';
import { Location } from '../../helpers/urls';
import './BubbleChart.css';
interface BubbleItem<T> {
    color?: string;
    key?: string;
    link?: string | Location;
    data?: T;
    size: number;
    tooltip?: React.ReactNode;
    x: number;
    y: number;
}
interface Props<T> {
    displayXGrid?: boolean;
    displayXTicks?: boolean;
    displayYGrid?: boolean;
    displayYTicks?: boolean;
    formatXTick: (tick: number) => string;
    formatYTick: (tick: number) => string;
    height: number;
    items: BubbleItem<T>[];
    onBubbleClick?: (ref?: T) => void;
    padding: [number, number, number, number];
    sizeDomain?: [number, number];
    sizeRange?: [number, number];
    xDomain?: [number, number];
    yDomain?: [number, number];
}
interface State {
    transform: {
        x: number;
        y: number;
        k: number;
    };
}
declare type Scale = ScaleLinear<number, number>;
export default class BubbleChart<T> extends React.PureComponent<Props<T>, State> {
    private node?;
    private zoom?;
    static defaultProps: {
        displayXGrid: boolean;
        displayXTicks: boolean;
        displayYGrid: boolean;
        displayYTicks: boolean;
        formatXTick: (d: number) => string;
        formatYTick: (d: number) => string;
        padding: number[];
        sizeRange: number[];
    };
    constructor(props: Props<T>);
    componentDidUpdate(): void;
    boundNode: (node: SVGSVGElement) => void;
    zoomed: () => void;
    resetZoom: (event: React.MouseEvent<Link, MouseEvent>) => void;
    getXRange(xScale: Scale, sizeScale: Scale, availableWidth: number): number[];
    getYRange(yScale: Scale, sizeScale: Scale, availableHeight: number): number[];
    getTicks(scale: Scale, format: (d: number) => string): number[];
    getZoomLevelLabel: () => string;
    renderXGrid: (ticks: number[], xScale: Scale, yScale: Scale) => JSX.Element | null;
    renderYGrid: (ticks: number[], xScale: Scale, yScale: Scale) => JSX.Element | null;
    renderXTicks: (xTicks: number[], xScale: Scale, yScale: Scale) => JSX.Element | null;
    renderYTicks: (yTicks: number[], xScale: Scale, yScale: Scale) => JSX.Element | null;
    renderChart: (width: number) => JSX.Element;
    render(): JSX.Element;
}
export {};
