"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var classNames = require("classnames");
var d3_array_1 = require("d3-array");
var d3_scale_1 = require("d3-scale");
var d3_selection_1 = require("d3-selection");
var d3_zoom_1 = require("d3-zoom");
var lodash_1 = require("lodash");
var React = require("react");
var react_router_1 = require("react-router");
var AutoSizer_1 = require("react-virtualized/dist/commonjs/AutoSizer");
var l10n_1 = require("../../helpers/l10n");
var Tooltip_1 = require("../controls/Tooltip");
require("./BubbleChart.css");
var TICKS_COUNT = 5;
var BubbleChart = (function (_super) {
    __extends(BubbleChart, _super);
    function BubbleChart(props) {
        var _this = _super.call(this, props) || this;
        _this.boundNode = function (node) {
            _this.node = node;
            _this.zoom = d3_zoom_1.zoom().scaleExtent([1, 10]).on('zoom', _this.zoomed);
            d3_selection_1.select(_this.node).call(_this.zoom);
        };
        _this.zoomed = function () {
            var padding = _this.props.padding;
            var _a = d3_selection_1.event.transform, x = _a.x, y = _a.y, k = _a.k;
            _this.setState({
                transform: {
                    x: x + padding[3] * (k - 1),
                    y: y + padding[0] * (k - 1),
                    k: k,
                },
            });
        };
        _this.resetZoom = function (event) {
            event.stopPropagation();
            event.preventDefault();
            if (_this.zoom && _this.node) {
                d3_selection_1.select(_this.node).call(_this.zoom.transform, d3_zoom_1.zoomIdentity);
            }
        };
        _this.getZoomLevelLabel = function () { return Math.floor(_this.state.transform.k * 100) + '%'; };
        _this.renderXGrid = function (ticks, xScale, yScale) {
            if (!_this.props.displayXGrid) {
                return null;
            }
            var transform = _this.state.transform;
            var lines = ticks.map(function (tick, index) {
                var x = xScale(tick);
                var y1 = yScale.range()[0];
                var y2 = yScale.range()[1];
                return (React.createElement("line", { className: "bubble-chart-grid", key: index, x1: x * transform.k + transform.x, x2: x * transform.k + transform.x, y1: y1 * transform.k, y2: transform.k > 1 ? 0 : y2 }));
            });
            return React.createElement("g", null, lines);
        };
        _this.renderYGrid = function (ticks, xScale, yScale) {
            if (!_this.props.displayYGrid) {
                return null;
            }
            var transform = _this.state.transform;
            var lines = ticks.map(function (tick, index) {
                var y = yScale(tick);
                var x1 = xScale.range()[0];
                var x2 = xScale.range()[1];
                return (React.createElement("line", { className: "bubble-chart-grid", key: index, x1: transform.k > 1 ? 0 : x1, x2: x2 * transform.k, y1: y * transform.k + transform.y, y2: y * transform.k + transform.y }));
            });
            return React.createElement("g", null, lines);
        };
        _this.renderXTicks = function (xTicks, xScale, yScale) {
            if (!_this.props.displayXTicks) {
                return null;
            }
            var transform = _this.state.transform;
            var ticks = xTicks.map(function (tick, index) {
                var x = xScale(tick) * transform.k + transform.x;
                var y = yScale.range()[0];
                var innerText = _this.props.formatXTick(tick);
                return x > 0 && x < xScale.range()[1] ? (React.createElement("text", { className: "bubble-chart-tick", dy: "1.5em", key: index, x: x, y: y }, innerText)) : null;
            });
            return React.createElement("g", null, ticks);
        };
        _this.renderYTicks = function (yTicks, xScale, yScale) {
            if (!_this.props.displayYTicks) {
                return null;
            }
            var transform = _this.state.transform;
            var ticks = yTicks.map(function (tick, index) {
                var x = xScale.range()[0];
                var y = yScale(tick) * transform.k + transform.y;
                var innerText = _this.props.formatYTick(tick);
                return y > 0 && y < yScale.range()[0] ? (React.createElement("text", { className: "bubble-chart-tick bubble-chart-tick-y", dx: "-0.5em", dy: "0.3em", key: index, x: x, y: y }, innerText)) : null;
            });
            return React.createElement("g", null, ticks);
        };
        _this.renderChart = function (width) {
            var transform = _this.state.transform;
            var availableWidth = width - _this.props.padding[1] - _this.props.padding[3];
            var availableHeight = _this.props.height - _this.props.padding[0] - _this.props.padding[2];
            var xScale = d3_scale_1.scaleLinear()
                .domain(_this.props.xDomain || [0, d3_array_1.max(_this.props.items, function (d) { return d.x; }) || 0])
                .range([0, availableWidth])
                .nice();
            var yScale = d3_scale_1.scaleLinear()
                .domain(_this.props.yDomain || [0, d3_array_1.max(_this.props.items, function (d) { return d.y; }) || 0])
                .range([availableHeight, 0])
                .nice();
            var sizeScale = d3_scale_1.scaleLinear()
                .domain(_this.props.sizeDomain || [0, d3_array_1.max(_this.props.items, function (d) { return d.size; }) || 0])
                .range(_this.props.sizeRange || []);
            var xScaleOriginal = xScale.copy();
            var yScaleOriginal = yScale.copy();
            xScale.range(_this.getXRange(xScale, sizeScale, availableWidth));
            yScale.range(_this.getYRange(yScale, sizeScale, availableHeight));
            var bubbles = lodash_1.sortBy(_this.props.items, function (b) { return -b.size; }).map(function (item, index) {
                return (React.createElement(Bubble, { color: item.color, data: item.data, key: item.key || index, link: item.link, onClick: _this.props.onBubbleClick, r: sizeScale(item.size), scale: 1 / transform.k, tooltip: item.tooltip, x: xScale(item.x), y: yScale(item.y) }));
            });
            var xTicks = _this.getTicks(xScale, _this.props.formatXTick);
            var yTicks = _this.getTicks(yScale, _this.props.formatYTick);
            return (React.createElement("svg", { className: classNames('bubble-chart'), height: _this.props.height, ref: _this.boundNode, width: width },
                React.createElement("defs", null,
                    React.createElement("clipPath", { id: "graph-region" },
                        React.createElement("rect", { height: availableHeight + 4, width: availableWidth + 4, x: -2, y: -2 }))),
                React.createElement("g", { transform: "translate(" + _this.props.padding[3] + ", " + _this.props.padding[0] + ")" },
                    React.createElement("g", { clipPath: "url(#graph-region)" },
                        _this.renderXGrid(xTicks, xScale, yScale),
                        _this.renderYGrid(yTicks, xScale, yScale),
                        React.createElement("g", { transform: "translate(" + transform.x + ", " + transform.y + ") scale(" + transform.k + ")" }, bubbles)),
                    _this.renderXTicks(xTicks, xScale, yScaleOriginal),
                    _this.renderYTicks(yTicks, xScaleOriginal, yScale))));
        };
        _this.state = { transform: { x: 0, y: 0, k: 1 } };
        return _this;
    }
    BubbleChart.prototype.componentDidUpdate = function () {
        if (this.zoom && this.node) {
            var rect = this.node.getBoundingClientRect();
            this.zoom.translateExtent([
                [0, 0],
                [rect.width, rect.height],
            ]);
        }
    };
    BubbleChart.prototype.getXRange = function (xScale, sizeScale, availableWidth) {
        var minX = d3_array_1.min(this.props.items, function (d) { return xScale(d.x) - sizeScale(d.size); }) || 0;
        var maxX = d3_array_1.max(this.props.items, function (d) { return xScale(d.x) + sizeScale(d.size); }) || 0;
        var dMinX = minX < 0 ? xScale.range()[0] - minX : xScale.range()[0];
        var dMaxX = maxX > xScale.range()[1] ? maxX - xScale.range()[1] : 0;
        return [dMinX, availableWidth - dMaxX];
    };
    BubbleChart.prototype.getYRange = function (yScale, sizeScale, availableHeight) {
        var minY = d3_array_1.min(this.props.items, function (d) { return yScale(d.y) - sizeScale(d.size); }) || 0;
        var maxY = d3_array_1.max(this.props.items, function (d) { return yScale(d.y) + sizeScale(d.size); }) || 0;
        var dMinY = minY < 0 ? yScale.range()[1] - minY : yScale.range()[1];
        var dMaxY = maxY > yScale.range()[0] ? maxY - yScale.range()[0] : 0;
        return [availableHeight - dMaxY, dMinY];
    };
    BubbleChart.prototype.getTicks = function (scale, format) {
        var zoom = Math.ceil(this.state.transform.k);
        var ticks = scale.ticks(TICKS_COUNT * zoom).map(function (tick) { return format(tick); });
        var uniqueTicksCount = lodash_1.uniq(ticks).length;
        var ticksCount = uniqueTicksCount < TICKS_COUNT * zoom ? uniqueTicksCount - 1 : TICKS_COUNT * zoom;
        return scale.ticks(ticksCount);
    };
    BubbleChart.prototype.render = function () {
        var _this = this;
        return (React.createElement("div", null,
            React.createElement("div", { className: "bubble-chart-zoom" },
                React.createElement(Tooltip_1.default, { overlay: l10n_1.translate('component_measures.bubble_chart.zoom_level') },
                    React.createElement(react_router_1.Link, { onClick: this.resetZoom, to: "#" }, this.getZoomLevelLabel()))),
            React.createElement(AutoSizer_1.AutoSizer, { disableHeight: true }, function (size) { return _this.renderChart(size.width); })));
    };
    BubbleChart.defaultProps = {
        displayXGrid: true,
        displayXTicks: true,
        displayYGrid: true,
        displayYTicks: true,
        formatXTick: function (d) { return String(d); },
        formatYTick: function (d) { return String(d); },
        padding: [10, 10, 10, 10],
        sizeRange: [5, 45],
    };
    return BubbleChart;
}(React.PureComponent));
exports.default = BubbleChart;
function Bubble(props) {
    var handleClick = function (event) {
        if (props.onClick) {
            event.stopPropagation();
            event.preventDefault();
            props.onClick(props.data);
        }
    };
    var circle = (React.createElement("circle", { className: "bubble-chart-bubble", onClick: props.onClick ? handleClick : undefined, r: props.r, style: { fill: props.color, stroke: props.color }, transform: "translate(" + props.x + ", " + props.y + ") scale(" + props.scale + ")" }));
    if (props.link && !props.onClick) {
        circle = React.createElement(react_router_1.Link, { to: props.link }, circle);
    }
    return (React.createElement(Tooltip_1.default, { overlay: props.tooltip || undefined },
        React.createElement("g", null, circle)));
}
//# sourceMappingURL=BubbleChart.js.map