"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var d3_shape_1 = require("d3-shape");
var React = require("react");
function DonutChart(props) {
    var height = props.height, _a = props.padding, padding = _a === void 0 ? [0, 0, 0, 0] : _a, width = props.width;
    var availableWidth = width - padding[1] - padding[3];
    var availableHeight = height - padding[0] - padding[2];
    var size = Math.min(availableWidth, availableHeight);
    var radius = Math.floor(size / 2);
    var pie = d3_shape_1.pie()
        .sort(null)
        .value(function (d) { return d.value; });
    if (props.padAngle !== undefined) {
        pie.padAngle(props.padAngle);
    }
    var sectors = pie(props.data).map(function (d, i) {
        return (React.createElement(Sector, { data: d, fill: props.data[i].fill, key: i, radius: radius, thickness: props.thickness }));
    });
    return (React.createElement("svg", { className: "donut-chart", height: height, width: width },
        React.createElement("g", { transform: "translate(" + padding[3] + ", " + padding[0] + ")" },
            React.createElement("g", { transform: "translate(" + radius + ", " + radius + ")" }, sectors))));
}
exports.default = DonutChart;
function Sector(props) {
    var arc = d3_shape_1.arc()
        .outerRadius(props.radius)
        .innerRadius(props.radius - props.thickness);
    var d = arc(props.data);
    return React.createElement("path", { d: d, style: { fill: props.fill } });
}
//# sourceMappingURL=DonutChart.js.map