"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var d3_array_1 = require("d3-array");
var d3_scale_1 = require("d3-scale");
var React = require("react");
var Tooltip_1 = require("../controls/Tooltip");
require("./BarChart.css");
require("./Histogram.css");
var BAR_HEIGHT = 10;
var DEFAULT_PADDING = [10, 10, 10, 10];
var Histogram = (function (_super) {
    __extends(Histogram, _super);
    function Histogram() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Histogram.prototype.renderBar = function (d, index, xScale, yScale) {
        var _a = this.props, alignTicks = _a.alignTicks, _b = _a.padding, padding = _b === void 0 ? DEFAULT_PADDING : _b;
        var width = Math.round(xScale(d)) + 1;
        var x = xScale.range()[0] + (alignTicks ? padding[3] : 0);
        var y = Math.round(yScale(index) + yScale.bandwidth() / 2);
        return React.createElement("rect", { className: "bar-chart-bar", height: BAR_HEIGHT, width: width, x: x, y: y });
    };
    Histogram.prototype.renderValue = function (d, index, xScale, yScale) {
        var _a = this.props, alignTicks = _a.alignTicks, _b = _a.padding, padding = _b === void 0 ? DEFAULT_PADDING : _b, yValues = _a.yValues;
        var value = yValues && yValues[index];
        if (!value) {
            return null;
        }
        var x = xScale(d) + (alignTicks ? padding[3] : 0);
        var y = Math.round(yScale(index) + yScale.bandwidth() / 2 + BAR_HEIGHT / 2);
        return (React.createElement(Tooltip_1.default, { overlay: this.props.yTooltips && this.props.yTooltips[index] },
            React.createElement("text", { className: "bar-chart-tick histogram-value", dx: "1em", dy: "0.3em", x: x, y: y }, value)));
    };
    Histogram.prototype.renderTick = function (index, xScale, yScale) {
        var _a = this.props, alignTicks = _a.alignTicks, yTicks = _a.yTicks;
        var tick = yTicks && yTicks[index];
        if (!tick) {
            return null;
        }
        var x = xScale.range()[0];
        var y = Math.round(yScale(index) + yScale.bandwidth() / 2 + BAR_HEIGHT / 2);
        var historyTickClass = alignTicks ? 'histogram-tick-start' : 'histogram-tick';
        return (React.createElement("text", { className: 'bar-chart-tick ' + historyTickClass, dx: alignTicks ? 0 : '-1em', dy: "0.3em", x: x, y: y }, tick));
    };
    Histogram.prototype.renderBars = function (xScale, yScale) {
        var _this = this;
        return (React.createElement("g", null, this.props.bars.map(function (d, index) {
            return (React.createElement("g", { key: index },
                _this.renderBar(d, index, xScale, yScale),
                _this.renderValue(d, index, xScale, yScale),
                _this.renderTick(index, xScale, yScale)));
        })));
    };
    Histogram.prototype.render = function () {
        var _a = this.props, bars = _a.bars, width = _a.width, height = _a.height, _b = _a.padding, padding = _b === void 0 ? DEFAULT_PADDING : _b;
        var availableWidth = width - padding[1] - padding[3];
        var xScale = d3_scale_1.scaleLinear()
            .domain([0, d3_array_1.max(bars)])
            .range([0, availableWidth]);
        var availableHeight = height - padding[0] - padding[2];
        var yScale = d3_scale_1.scaleBand()
            .domain(bars.map(function (_, index) { return index; }))
            .rangeRound([0, availableHeight]);
        return (React.createElement("svg", { className: "bar-chart", height: this.props.height, width: this.props.width },
            React.createElement("g", { transform: "translate(" + (this.props.alignTicks ? 4 : padding[3]) + ", " + padding[0] + ")" }, this.renderBars(xScale, yScale))));
    };
    return Histogram;
}(React.PureComponent));
exports.default = Histogram;
//# sourceMappingURL=Histogram.js.map