"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var d3_array_1 = require("d3-array");
var d3_scale_1 = require("d3-scale");
var d3_shape_1 = require("d3-shape");
var React = require("react");
var AutoSizer_1 = require("react-virtualized/dist/commonjs/AutoSizer");
require("./LineChart.css");
var LineChart = (function (_super) {
    __extends(LineChart, _super);
    function LineChart() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.renderChart = function (width) {
            var _a = _this.props, height = _a.height, _b = _a.padding, padding = _b === void 0 ? [10, 10, 10, 10] : _b;
            if (!width || !height) {
                return React.createElement("div", null);
            }
            var availableWidth = width - padding[1] - padding[3];
            var availableHeight = height - padding[0] - padding[2];
            var xScale = d3_scale_1.scaleLinear()
                .domain(d3_array_1.extent(_this.props.data, function (d) { return d.x; }))
                .range([0, availableWidth]);
            var yScale = d3_scale_1.scaleLinear().range([availableHeight, 0]);
            if (_this.props.domain) {
                yScale.domain(_this.props.domain);
            }
            else {
                var maxY = d3_array_1.max(_this.props.data, function (d) { return d.y; });
                yScale.domain([0, maxY]);
            }
            return (React.createElement("svg", { className: "line-chart", height: height, width: width },
                React.createElement("g", { transform: "translate(" + padding[3] + ", " + padding[0] + ")" },
                    _this.renderVerticalGrid(xScale, yScale),
                    _this.renderBackdrop(xScale, yScale),
                    _this.renderLine(xScale, yScale),
                    _this.renderPoints(xScale, yScale),
                    _this.renderXTicks(xScale, yScale),
                    _this.renderXValues(xScale, yScale))));
        };
        return _this;
    }
    LineChart.prototype.renderBackdrop = function (xScale, yScale) {
        var _a = this.props.displayBackdrop, displayBackdrop = _a === void 0 ? true : _a;
        if (!displayBackdrop) {
            return null;
        }
        var area = d3_shape_1.area()
            .x(function (d) { return xScale(d.x); })
            .y0(yScale.range()[0])
            .y1(function (d) { return yScale(d.y || 0); })
            .defined(function (d) { return d.y != null; })
            .curve(d3_shape_1.curveBasis);
        var data = this.props.data;
        if (this.props.backdropConstraints) {
            var c_1 = this.props.backdropConstraints;
            data = data.filter(function (d) { return c_1[0] <= d.x && d.x <= c_1[1]; });
        }
        return React.createElement("path", { className: "line-chart-backdrop", d: area(data) });
    };
    LineChart.prototype.renderPoints = function (xScale, yScale) {
        var _a = this.props.displayPoints, displayPoints = _a === void 0 ? true : _a;
        if (!displayPoints) {
            return null;
        }
        var points = this.props.data
            .filter(function (point) { return point.y != null; })
            .map(function (point, index) {
            var x = xScale(point.x);
            var y = yScale(point.y || 0);
            return React.createElement("circle", { className: "line-chart-point", cx: x, cy: y, key: index, r: "3" });
        });
        return React.createElement("g", null, points);
    };
    LineChart.prototype.renderVerticalGrid = function (xScale, yScale) {
        var _a = this.props.displayVerticalGrid, displayVerticalGrid = _a === void 0 ? true : _a;
        if (!displayVerticalGrid) {
            return null;
        }
        var lines = this.props.data.map(function (point, index) {
            var x = xScale(point.x);
            var y1 = yScale.range()[0];
            var y2 = yScale(point.y || 0);
            return React.createElement("line", { className: "line-chart-grid", key: index, x1: x, x2: x, y1: y1, y2: y2 });
        });
        return React.createElement("g", null, lines);
    };
    LineChart.prototype.renderXTicks = function (xScale, yScale) {
        var _this = this;
        var _a = this.props.xTicks, xTicks = _a === void 0 ? [] : _a;
        if (!xTicks.length) {
            return null;
        }
        var ticks = xTicks.map(function (tick, index) {
            var point = _this.props.data[index];
            var x = xScale(point.x);
            var y = yScale.range()[0];
            return (React.createElement("text", { className: "line-chart-tick", dy: "1.5em", key: index, x: x, y: y }, tick));
        });
        return React.createElement("g", null, ticks);
    };
    LineChart.prototype.renderXValues = function (xScale, yScale) {
        var _this = this;
        var _a = this.props.xValues, xValues = _a === void 0 ? [] : _a;
        if (!xValues.length) {
            return null;
        }
        var ticks = xValues.map(function (value, index) {
            var point = _this.props.data[index];
            var x = xScale(point.x);
            var y = yScale(point.y || 0);
            return (React.createElement("text", { className: "line-chart-tick", dy: "-1em", key: index, x: x, y: y }, value));
        });
        return React.createElement("g", null, ticks);
    };
    LineChart.prototype.renderLine = function (xScale, yScale) {
        var p = d3_shape_1.line()
            .x(function (d) { return xScale(d.x); })
            .y(function (d) { return yScale(d.y || 0); })
            .defined(function (d) { return d.y != null; })
            .curve(d3_shape_1.curveBasis);
        return React.createElement("path", { className: "line-chart-path", d: p(this.props.data) });
    };
    LineChart.prototype.render = function () {
        var _this = this;
        return this.props.width !== undefined ? (this.renderChart(this.props.width)) : (React.createElement(AutoSizer_1.AutoSizer, { disableHeight: true }, function (size) { return _this.renderChart(size.width); }));
    };
    return LineChart;
}(React.PureComponent));
exports.default = LineChart;
//# sourceMappingURL=LineChart.js.map