"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var classNames = require("classnames");
var d3_array_1 = require("d3-array");
var d3_scale_1 = require("d3-scale");
var d3_shape_1 = require("d3-shape");
var lodash_1 = require("lodash");
var React = require("react");
var react_draggable_1 = require("react-draggable");
var theme_1 = require("../theme");
require("./LineChart.css");
require("./ZoomTimeLine.css");
var ZoomTimeLine = (function (_super) {
    __extends(ZoomTimeLine, _super);
    function ZoomTimeLine(props) {
        var _this = _super.call(this, props) || this;
        _this.getRatingScale = function (availableHeight) {
            return d3_scale_1.scalePoint().domain([5, 4, 3, 2, 1]).range([availableHeight, 0]);
        };
        _this.getLevelScale = function (availableHeight) {
            return d3_scale_1.scalePoint().domain(['ERROR', 'WARN', 'OK']).range([availableHeight, 0]);
        };
        _this.getYScale = function (availableHeight, flatData) {
            if (_this.props.metricType === 'RATING') {
                return _this.getRatingScale(availableHeight);
            }
            else if (_this.props.metricType === 'LEVEL') {
                return _this.getLevelScale(availableHeight);
            }
            else {
                return d3_scale_1.scaleLinear()
                    .range([availableHeight, 0])
                    .domain([0, d3_array_1.max(flatData, function (d) { return Number(d.y || 0); })])
                    .nice();
            }
        };
        _this.getXScale = function (availableWidth, flatData) {
            return d3_scale_1.scaleTime()
                .domain(d3_array_1.extent(flatData, function (d) { return d.x; }))
                .range([0, availableWidth])
                .clamp(true);
        };
        _this.getScales = function () {
            var availableWidth = _this.props.width - _this.props.padding[1] - _this.props.padding[3];
            var availableHeight = _this.props.height - _this.props.padding[0] - _this.props.padding[2];
            var flatData = lodash_1.flatten(_this.props.series.map(function (serie) { return serie.data; }));
            return {
                xScale: _this.getXScale(availableWidth, flatData),
                yScale: _this.getYScale(availableHeight, flatData),
            };
        };
        _this.getEventMarker = function (size) {
            var half = size / 2;
            return "M" + half + " 0 L" + size + " " + half + " L " + half + " " + size + " L0 " + half + " L" + half + " 0 L" + size + " " + half;
        };
        _this.handleDoubleClick = function (xScale, xDim) { return function () {
            _this.handleZoomUpdate(xScale, xDim);
        }; };
        _this.handleSelectionDrag = function (xScale, width, xDim, checkDelta) { return function (_, data) {
            if (!checkDelta || data.deltaX) {
                var x = Math.max(xDim[0], Math.min(data.x, xDim[1] - width));
                _this.handleZoomUpdate(xScale, [x, width + x]);
            }
        }; };
        _this.handleSelectionHandleDrag = function (xScale, fixedX, xDim, handleDirection, checkDelta) { return function (_, data) {
            if (!checkDelta || data.deltaX) {
                var x = Math.max(xDim[0], Math.min(data.x, xDim[1]));
                _this.handleZoomUpdate(xScale, handleDirection === 'right' ? [fixedX, x] : [x, fixedX]);
            }
        }; };
        _this.handleNewZoomDragStart = function (xDim) { return function (_, data) {
            var overlayLeftPos = data.node.getBoundingClientRect().left;
            _this.setState({
                overlayLeftPos: overlayLeftPos,
                newZoomStart: Math.round(Math.max(xDim[0], Math.min(data.x - overlayLeftPos, xDim[1]))),
            });
        }; };
        _this.handleNewZoomDrag = function (xScale, xDim) { return function (_, data) {
            var _a = _this.state, newZoomStart = _a.newZoomStart, overlayLeftPos = _a.overlayLeftPos;
            if (newZoomStart != null && overlayLeftPos != null && data.deltaX) {
                _this.handleZoomUpdate(xScale, lodash_1.sortBy([newZoomStart, Math.max(xDim[0], Math.min(data.x - overlayLeftPos, xDim[1]))]));
            }
        }; };
        _this.handleNewZoomDragEnd = function (xScale, xDim) { return function (_, data) {
            var _a = _this.state, newZoomStart = _a.newZoomStart, overlayLeftPos = _a.overlayLeftPos;
            if (newZoomStart !== undefined && overlayLeftPos !== undefined) {
                var x = Math.round(Math.max(xDim[0], Math.min(data.x - overlayLeftPos, xDim[1])));
                _this.handleZoomUpdate(xScale, newZoomStart === x ? xDim : lodash_1.sortBy([newZoomStart, x]));
                _this.setState({ newZoomStart: undefined, overlayLeftPos: undefined });
            }
        }; };
        _this.handleZoomUpdate = function (xScale, xArray) {
            var xRange = xScale.range();
            var startDate = xArray[0] > xRange[0] && xArray[0] < xRange[xRange.length - 1]
                ? xScale.invert(xArray[0])
                : undefined;
            var endDate = xArray[1] > xRange[0] && xArray[1] < xRange[xRange.length - 1]
                ? xScale.invert(xArray[1])
                : undefined;
            if (_this.props.startDate !== startDate || _this.props.endDate !== endDate) {
                _this.props.updateZoom(startDate, endDate);
            }
        };
        _this.renderBaseLine = function (xScale, yScale) {
            return (React.createElement("line", { className: "line-chart-grid", x1: xScale.range()[0], x2: xScale.range()[1], y1: yScale.range()[0], y2: yScale.range()[0] }));
        };
        _this.renderTicks = function (xScale, yScale) {
            var format = xScale.tickFormat(7);
            var ticks = xScale.ticks(7);
            var y = yScale.range()[0];
            return (React.createElement("g", null, ticks.slice(0, -1).map(function (tick, index) {
                var nextTick = index + 1 < ticks.length ? ticks[index + 1] : xScale.domain()[1];
                var x = (xScale(tick) + xScale(nextTick)) / 2;
                return (React.createElement("text", { className: "chart-zoom-tick", dy: "1.3em", key: index, x: x, y: y }, format(tick)));
            })));
        };
        _this.renderLeak = function (xScale, yScale) {
            var leakPeriodDate = _this.props.leakPeriodDate;
            if (!leakPeriodDate) {
                return null;
            }
            var yRange = yScale.range();
            return (React.createElement(theme_1.ThemeConsumer, null, function (theme) { return (React.createElement("rect", { fill: theme.colors.leakPrimaryColor, height: yRange[0] - yRange[yRange.length - 1], width: xScale.range()[1] - xScale(leakPeriodDate), x: xScale(leakPeriodDate), y: yRange[yRange.length - 1] })); }));
        };
        _this.renderLines = function (xScale, yScale) {
            var lineGenerator = d3_shape_1.line()
                .defined(function (d) { return Boolean(d.y || d.y === 0); })
                .x(function (d) { return xScale(d.x); })
                .y(function (d) { return yScale(d.y); });
            if (_this.props.basisCurve) {
                lineGenerator.curve(d3_shape_1.curveBasis);
            }
            return (React.createElement("g", null, _this.props.series.map(function (serie, idx) { return (React.createElement("path", { className: classNames('line-chart-path', 'line-chart-path-' + idx), d: lineGenerator(serie.data) || undefined, key: serie.name })); })));
        };
        _this.renderAreas = function (xScale, yScale) {
            var areaGenerator = d3_shape_1.area()
                .defined(function (d) { return Boolean(d.y || d.y === 0); })
                .x(function (d) { return xScale(d.x); })
                .y1(function (d) { return yScale(d.y); })
                .y0(yScale(0));
            if (_this.props.basisCurve) {
                areaGenerator.curve(d3_shape_1.curveBasis);
            }
            return (React.createElement("g", null, _this.props.series.map(function (serie, idx) { return (React.createElement("path", { className: classNames('line-chart-area', 'line-chart-area-' + idx), d: areaGenerator(serie.data) || undefined, key: serie.name })); })));
        };
        _this.renderZoomHandle = function (options) { return (React.createElement(react_draggable_1.default, { axis: "x", bounds: { left: options.xDim[0], right: options.xDim[1] }, onDrag: _this.handleSelectionHandleDrag(options.xScale, options.fixedPos, options.xDim, options.direction, true), onStop: _this.handleSelectionHandleDrag(options.xScale, options.fixedPos, options.xDim, options.direction), position: { x: options.xPos, y: 0 } },
            React.createElement("rect", { className: "zoom-selection-handle", height: options.yDim[0] - options.yDim[1] + 1, width: 6, x: -3, y: options.yDim[1] }))); };
        _this.renderZoom = function (xScale, yScale) {
            var xRange = xScale.range();
            var yRange = yScale.range();
            var xDim = [xRange[0], xRange[xRange.length - 1]];
            var yDim = [yRange[0], yRange[yRange.length - 1]];
            var startX = Math.round(_this.props.startDate ? xScale(_this.props.startDate) : xDim[0]);
            var endX = Math.round(_this.props.endDate ? xScale(_this.props.endDate) : xDim[1]);
            var xArray = lodash_1.sortBy([startX, endX]);
            var zoomBoxWidth = xArray[1] - xArray[0];
            var showZoomArea = _this.state.newZoomStart == null ||
                _this.state.newZoomStart === startX ||
                _this.state.newZoomStart === endX;
            return (React.createElement("g", { className: "chart-zoom" },
                React.createElement(react_draggable_1.DraggableCore, { onDrag: _this.handleNewZoomDrag(xScale, xDim), onStart: _this.handleNewZoomDragStart(xDim), onStop: _this.handleNewZoomDragEnd(xScale, xDim) },
                    React.createElement("rect", { className: "zoom-overlay", height: yDim[0] - yDim[1], width: xDim[1] - xDim[0], x: xDim[0], y: yDim[1] })),
                showZoomArea && (React.createElement(react_draggable_1.default, { axis: "x", bounds: { left: xDim[0], right: Math.floor(xDim[1] - zoomBoxWidth) }, onDrag: _this.handleSelectionDrag(xScale, zoomBoxWidth, xDim, true), onStop: _this.handleSelectionDrag(xScale, zoomBoxWidth, xDim), position: { x: xArray[0], y: 0 } },
                    React.createElement("rect", { className: "zoom-selection", height: yDim[0] - yDim[1] + 1, onDoubleClick: _this.handleDoubleClick(xScale, xDim), width: zoomBoxWidth, x: 0, y: yDim[1] }))),
                showZoomArea &&
                    _this.renderZoomHandle({
                        xScale: xScale,
                        xPos: startX,
                        fixedPos: endX,
                        xDim: xDim,
                        yDim: yDim,
                        direction: 'left',
                    }),
                showZoomArea &&
                    _this.renderZoomHandle({
                        xScale: xScale,
                        xPos: endX,
                        fixedPos: startX,
                        xDim: xDim,
                        yDim: yDim,
                        direction: 'right',
                    })));
        };
        _this.state = {};
        _this.handleZoomUpdate = lodash_1.throttle(_this.handleZoomUpdate, 40);
        return _this;
    }
    ZoomTimeLine.prototype.render = function () {
        if (!this.props.width || !this.props.height) {
            return React.createElement("div", null);
        }
        var _a = this.getScales(), xScale = _a.xScale, yScale = _a.yScale;
        return (React.createElement("svg", { className: "line-chart ", height: this.props.height, width: this.props.width },
            React.createElement("g", { transform: "translate(" + this.props.padding[3] + ", " + (this.props.padding[0] + 2) + ")" },
                this.renderLeak(xScale, yScale),
                this.renderBaseLine(xScale, yScale),
                this.props.showXTicks && this.renderTicks(xScale, yScale),
                this.props.showAreas && this.renderAreas(xScale, yScale),
                this.renderLines(xScale, yScale),
                this.renderZoom(xScale, yScale))));
    };
    ZoomTimeLine.defaultProps = {
        padding: [0, 0, 18, 0],
        showXTicks: true,
    };
    return ZoomTimeLine;
}(React.PureComponent));
exports.default = ZoomTimeLine;
//# sourceMappingURL=ZoomTimeLine.js.map