"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var classNames = require("classnames");
var lodash_1 = require("lodash");
var React = require("react");
var l10n_1 = require("../../helpers/l10n");
var SearchIcon_1 = require("../icons/SearchIcon");
var DeferredSpinner_1 = require("../ui/DeferredSpinner");
var buttons_1 = require("./buttons");
require("./SearchBox.css");
var DEFAULT_MAX_LENGTH = 100;
var SearchBox = (function (_super) {
    __extends(SearchBox, _super);
    function SearchBox(props) {
        var _this = _super.call(this, props) || this;
        _this.changeValue = function (value, debounced) {
            if (debounced === void 0) { debounced = true; }
            var minLength = _this.props.minLength;
            if (value.length === 0) {
                _this.props.onChange('');
                _this.debouncedOnChange.cancel();
            }
            else if (!minLength || minLength <= value.length) {
                if (debounced) {
                    _this.debouncedOnChange(value);
                }
                else {
                    _this.props.onChange(value);
                }
            }
        };
        _this.handleInputChange = function (event) {
            var value = event.currentTarget.value;
            _this.setState({ value: value });
            _this.changeValue(value);
        };
        _this.handleInputKeyDown = function (event) {
            if (event.keyCode === 27) {
                event.preventDefault();
                _this.handleResetClick();
            }
            if (_this.props.onKeyDown) {
                _this.props.onKeyDown(event);
            }
        };
        _this.handleResetClick = function () {
            _this.changeValue('', false);
            if (_this.props.value === undefined || _this.props.value === '') {
                _this.setState({ value: '' });
            }
            if (_this.input) {
                _this.input.focus();
            }
        };
        _this.ref = function (node) {
            _this.input = node;
            if (_this.props.innerRef) {
                _this.props.innerRef(node);
            }
        };
        _this.state = { value: props.value || '' };
        _this.debouncedOnChange = lodash_1.debounce(_this.props.onChange, 250);
        return _this;
    }
    SearchBox.prototype.componentDidUpdate = function (prevProps) {
        if (this.props.value !== undefined &&
            this.state.value === prevProps.value &&
            this.state.value !== this.props.value) {
            this.setState({ value: this.props.value });
        }
    };
    SearchBox.prototype.render = function () {
        var _a = this.props, loading = _a.loading, minLength = _a.minLength, _b = _a.maxLength, maxLength = _b === void 0 ? DEFAULT_MAX_LENGTH : _b;
        var value = this.state.value;
        var inputClassName = classNames('search-box-input', {
            touched: value.length > 0 && (!minLength || minLength > value.length),
        });
        var tooShort = minLength !== undefined && value.length > 0 && value.length < minLength;
        return (React.createElement("div", { className: classNames('search-box', this.props.className), id: this.props.id, title: tooShort ? l10n_1.translateWithParameters('select2.tooShort', minLength) : '' },
            React.createElement("input", { "aria-label": l10n_1.translate('search_verb'), autoComplete: "off", autoFocus: this.props.autoFocus, className: inputClassName, maxLength: maxLength, onChange: this.handleInputChange, onClick: this.props.onClick, onFocus: this.props.onFocus, onKeyDown: this.handleInputKeyDown, placeholder: this.props.placeholder, ref: this.ref, type: "search", value: value }),
            React.createElement(DeferredSpinner_1.default, { loading: loading !== undefined ? loading : false },
                React.createElement(SearchIcon_1.default, { className: "search-box-magnifier" })),
            value && (React.createElement(buttons_1.ClearButton, { "aria-label": l10n_1.translate('clear'), className: "button-tiny search-box-clear", iconProps: { size: 12 }, onClick: this.handleResetClick })),
            tooShort && (React.createElement("span", { className: "search-box-note" }, l10n_1.translateWithParameters('select2.tooShort', minLength)))));
    };
    return SearchBox;
}(React.PureComponent));
exports.default = SearchBox;
//# sourceMappingURL=SearchBox.js.map