"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var classNames = require("classnames");
var lodash_1 = require("lodash");
var React = require("react");
var react_dom_1 = require("react-dom");
var ClickEventBoundary_1 = require("../controls/ClickEventBoundary");
var ScreenPositionFixer_1 = require("../controls/ScreenPositionFixer");
require("./popups.css");
var PopupPlacement;
(function (PopupPlacement) {
    PopupPlacement["Bottom"] = "bottom";
    PopupPlacement["BottomLeft"] = "bottom-left";
    PopupPlacement["BottomRight"] = "bottom-right";
    PopupPlacement["LeftTop"] = "left-top";
    PopupPlacement["RightTop"] = "right-top";
    PopupPlacement["RightBottom"] = "right-bottom";
    PopupPlacement["TopLeft"] = "top-left";
})(PopupPlacement = exports.PopupPlacement || (exports.PopupPlacement = {}));
function PopupBase(props, ref) {
    var _a = props.placement, placement = _a === void 0 ? PopupPlacement.Bottom : _a;
    return (React.createElement(ClickEventBoundary_1.default, null,
        React.createElement("div", { className: classNames('popup', "is-" + placement, { 'no-padding': props.noPadding }, props.className), ref: ref || React.createRef(), style: props.style },
            props.children,
            React.createElement(PopupArrow, { style: props.arrowStyle }))));
}
var PopupWithRef = React.forwardRef(PopupBase);
PopupWithRef.displayName = 'Popup';
exports.Popup = PopupWithRef;
function PopupArrow(props) {
    return React.createElement("div", { className: "popup-arrow", style: props.style });
}
exports.PopupArrow = PopupArrow;
function isMeasured(state) {
    return state.height !== undefined;
}
var PortalPopup = (function (_super) {
    __extends(PortalPopup, _super);
    function PortalPopup(props) {
        var _this = _super.call(this, props) || this;
        _this.mounted = false;
        _this.popupNode = React.createRef();
        _this.addEventListeners = function () {
            window.addEventListener('resize', _this.throttledPositionTooltip);
            window.addEventListener('scroll', _this.throttledPositionTooltip);
        };
        _this.removeEventListeners = function () {
            window.removeEventListener('resize', _this.throttledPositionTooltip);
            window.removeEventListener('scroll', _this.throttledPositionTooltip);
        };
        _this.getPlacement = function () {
            return _this.props.placement || PopupPlacement.Bottom;
        };
        _this.adjustArrowPosition = function (placement, _a) {
            var leftFix = _a.leftFix, topFix = _a.topFix;
            var _b = _this.props.arrowOffset, arrowOffset = _b === void 0 ? 0 : _b;
            switch (placement) {
                case PopupPlacement.Bottom:
                case PopupPlacement.BottomLeft:
                case PopupPlacement.BottomRight:
                case PopupPlacement.TopLeft:
                    return { marginLeft: -leftFix + arrowOffset };
                default:
                    return { marginTop: -topFix + arrowOffset };
            }
        };
        _this.positionPopup = function () {
            var toggleNode = react_dom_1.findDOMNode(_this);
            if (toggleNode && toggleNode instanceof Element && _this.popupNode.current) {
                var toggleRect = toggleNode.getBoundingClientRect();
                var _a = _this.popupNode.current.getBoundingClientRect(), width = _a.width, height = _a.height;
                var left = 0;
                var top_1 = 0;
                switch (_this.getPlacement()) {
                    case PopupPlacement.Bottom:
                        left = toggleRect.left + toggleRect.width / 2 - width / 2;
                        top_1 = toggleRect.top + toggleRect.height;
                        break;
                    case PopupPlacement.BottomLeft:
                        left = toggleRect.left;
                        top_1 = toggleRect.top + toggleRect.height;
                        break;
                    case PopupPlacement.BottomRight:
                        left = toggleRect.left + toggleRect.width - width;
                        top_1 = toggleRect.top + toggleRect.height;
                        break;
                    case PopupPlacement.LeftTop:
                        left = toggleRect.left - width;
                        top_1 = toggleRect.top;
                        break;
                    case PopupPlacement.RightTop:
                        left = toggleRect.left + toggleRect.width;
                        top_1 = toggleRect.top;
                        break;
                    case PopupPlacement.RightBottom:
                        left = toggleRect.left + toggleRect.width;
                        top_1 = toggleRect.top + toggleRect.height - height;
                        break;
                    case PopupPlacement.TopLeft:
                        left = toggleRect.left;
                        top_1 = toggleRect.top - height;
                        break;
                }
                _this.setState({
                    left: window.pageXOffset + left,
                    top: window.pageYOffset + top_1,
                    width: width,
                    height: height,
                });
            }
        };
        _this.renderActual = function (_a) {
            var _b = _a.leftFix, leftFix = _b === void 0 ? 0 : _b, _c = _a.topFix, topFix = _c === void 0 ? 0 : _c;
            var _d = _this.props, className = _d.className, overlay = _d.overlay, noPadding = _d.noPadding;
            var placement = _this.getPlacement();
            var arrowStyle;
            var style;
            if (isMeasured(_this.state)) {
                style = {
                    left: _this.state.left + leftFix,
                    top: _this.state.top + topFix,
                    width: _this.state.width,
                    height: _this.state.height,
                };
                arrowStyle = _this.adjustArrowPosition(placement, { leftFix: leftFix, topFix: topFix });
            }
            return (React.createElement(exports.Popup, { arrowStyle: arrowStyle, className: className, noPadding: noPadding, placement: placement, ref: _this.popupNode, style: style }, overlay));
        };
        _this.state = {};
        _this.throttledPositionTooltip = lodash_1.throttle(_this.positionPopup, 10);
        return _this;
    }
    PortalPopup.prototype.componentDidMount = function () {
        this.mounted = true;
        this.positionPopup();
        this.addEventListeners();
    };
    PortalPopup.prototype.componentDidUpdate = function (prevProps) {
        if (this.props.placement !== prevProps.placement || this.props.overlay !== prevProps.overlay) {
            this.positionPopup();
        }
    };
    PortalPopup.prototype.componentWillUnmount = function () {
        this.mounted = false;
        this.removeEventListeners();
    };
    PortalPopup.prototype.render = function () {
        return (React.createElement(React.Fragment, null,
            this.props.children,
            this.props.overlay && (React.createElement(PortalWrapper, null,
                React.createElement(ScreenPositionFixer_1.default, { ready: isMeasured(this.state) }, this.renderActual)))));
    };
    return PortalPopup;
}(React.Component));
exports.PortalPopup = PortalPopup;
var PortalWrapper = (function (_super) {
    __extends(PortalWrapper, _super);
    function PortalWrapper(props) {
        var _this = _super.call(this, props) || this;
        _this.el = document.createElement('div');
        _this.el.classList.add('popup-portal');
        return _this;
    }
    PortalWrapper.prototype.componentDidMount = function () {
        document.body.appendChild(this.el);
    };
    PortalWrapper.prototype.componentWillUnmount = function () {
        document.body.removeChild(this.el);
    };
    PortalWrapper.prototype.render = function () {
        return react_dom_1.createPortal(this.props.children, this.el);
    };
    return PortalWrapper;
}(React.Component));
//# sourceMappingURL=popups.js.map