"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var dates_1 = require("./dates");
function queriesEqual(a, b) {
    var keysA = Object.keys(a);
    var keysB = Object.keys(b);
    if (keysA.length !== keysB.length) {
        return false;
    }
    return keysA.every(function (key) { return lodash_1.isEqual(a[key], b[key]); });
}
exports.queriesEqual = queriesEqual;
function cleanQuery(query) {
    return lodash_1.omitBy(query, lodash_1.isNil);
}
exports.cleanQuery = cleanQuery;
function parseAsBoolean(value, defaultValue) {
    if (defaultValue === void 0) { defaultValue = true; }
    if (value === 'false') {
        return false;
    }
    if (value === 'true') {
        return true;
    }
    return defaultValue;
}
exports.parseAsBoolean = parseAsBoolean;
function parseAsOptionalBoolean(value) {
    if (value === 'true') {
        return true;
    }
    else if (value === 'false') {
        return false;
    }
    else {
        return undefined;
    }
}
exports.parseAsOptionalBoolean = parseAsOptionalBoolean;
function parseAsDate(value) {
    if (value) {
        var date = dates_1.parseDate(value);
        if (dates_1.isValidDate(date)) {
            return date;
        }
    }
    return undefined;
}
exports.parseAsDate = parseAsDate;
function parseAsString(value) {
    return value || '';
}
exports.parseAsString = parseAsString;
function parseAsOptionalString(value) {
    return value || undefined;
}
exports.parseAsOptionalString = parseAsOptionalString;
function parseAsArray(value, itemParser) {
    return value ? value.split(',').map(itemParser) : [];
}
exports.parseAsArray = parseAsArray;
function parseAsOptionalArray(value, itemParser) {
    return value ? parseAsArray(value, itemParser) : undefined;
}
exports.parseAsOptionalArray = parseAsOptionalArray;
function serializeDate(value, serializer) {
    if (serializer === void 0) { serializer = dates_1.toNotSoISOString; }
    if (value != null && value.toISOString) {
        return serializer(value);
    }
    return undefined;
}
exports.serializeDate = serializeDate;
function serializeDateShort(value) {
    return serializeDate(value, dates_1.toShortNotSoISOString);
}
exports.serializeDateShort = serializeDateShort;
function serializeString(value) {
    return value || undefined;
}
exports.serializeString = serializeString;
function serializeStringArray(value) {
    return value && value.length ? value.join() : undefined;
}
exports.serializeStringArray = serializeStringArray;
function serializeOptionalBoolean(value) {
    if (value === true) {
        return 'true';
    }
    else if (value === false) {
        return 'false';
    }
    else {
        return undefined;
    }
}
exports.serializeOptionalBoolean = serializeOptionalBoolean;
//# sourceMappingURL=query.js.map