"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var SCROLLING_DURATION = 100;
var SCROLLING_INTERVAL = 10;
var SCROLLING_STEPS = SCROLLING_DURATION / SCROLLING_INTERVAL;
function isWindow(element) {
    return element === window;
}
function getScroll(element) {
    return isWindow(element)
        ? { x: window.pageXOffset, y: window.pageYOffset }
        : { x: element.scrollLeft, y: element.scrollTop };
}
function scrollElement(element, x, y) {
    if (isWindow(element)) {
        window.scrollTo(x, y);
    }
    else {
        element.scrollLeft = x;
        element.scrollTop = y;
    }
    return Promise.resolve();
}
function smoothScroll(target, current, scroll) {
    var positiveDirection = target > current;
    var step = Math.ceil(Math.abs(target - current) / SCROLLING_STEPS);
    var stepsDone = 0;
    return new Promise(function (resolve) {
        var interval = setInterval(function () {
            if (current === target || SCROLLING_STEPS === stepsDone) {
                clearInterval(interval);
                resolve();
            }
            else {
                var goal = void 0;
                if (positiveDirection) {
                    goal = Math.min(target, current + step);
                }
                else {
                    goal = Math.max(target, current - step);
                }
                stepsDone++;
                current = goal;
                scroll(goal);
            }
        }, SCROLLING_INTERVAL);
    });
}
function smoothScrollTop(parent, position) {
    var scroll = getScroll(parent);
    return smoothScroll(position, scroll.y, function (position) { return scrollElement(parent, scroll.x, position); });
}
function smoothScrollLeft(parent, position) {
    var scroll = getScroll(parent);
    return smoothScroll(position, scroll.x, function (position) { return scrollElement(parent, position, scroll.y); });
}
function scrollToElement(element, options) {
    var opts = __assign({ topOffset: 0, bottomOffset: 0, parent: window, smooth: true }, options);
    var parent = opts.parent;
    var _a = element.getBoundingClientRect(), top = _a.top, bottom = _a.bottom;
    var scroll = getScroll(parent);
    var height = isWindow(parent)
        ? window.innerHeight
        : parent.getBoundingClientRect().height;
    var parentTop = isWindow(parent) ? 0 : parent.getBoundingClientRect().top;
    if (top - parentTop < opts.topOffset) {
        var goal = scroll.y - opts.topOffset + top - parentTop;
        if (opts.smooth) {
            addToScrollQueue(smoothScrollTop, parent, goal);
        }
        else {
            addToScrollQueue(scrollElement, parent, scroll.x, goal);
        }
    }
    if (bottom - parentTop > height - opts.bottomOffset) {
        var goal = scroll.y + bottom - parentTop - height + opts.bottomOffset;
        if (opts.smooth) {
            addToScrollQueue(smoothScrollTop, parent, goal);
        }
        else {
            addToScrollQueue(scrollElement, parent, scroll.x, goal);
        }
    }
}
exports.scrollToElement = scrollToElement;
function scrollHorizontally(element, options) {
    var opts = __assign({ leftOffset: 0, rightOffset: 0, parent: window, smooth: true }, options);
    var parent = opts.parent;
    var _a = element.getBoundingClientRect(), left = _a.left, right = _a.right;
    var scroll = getScroll(parent);
    var _b = isWindow(parent)
        ? { left: 0, width: window.innerWidth }
        : parent.getBoundingClientRect(), parentLeft = _b.left, width = _b.width;
    if (left - parentLeft < opts.leftOffset) {
        var goal = scroll.x - opts.leftOffset + left - parentLeft;
        if (opts.smooth) {
            addToScrollQueue(smoothScrollLeft, parent, goal);
        }
        else {
            addToScrollQueue(scrollElement, parent, goal, scroll.y);
        }
    }
    if (right - parentLeft > width - opts.rightOffset) {
        var goal = scroll.x + right - parentLeft - width + opts.rightOffset;
        if (opts.smooth) {
            addToScrollQueue(smoothScrollLeft, parent, goal);
        }
        else {
            addToScrollQueue(scrollElement, parent, goal, scroll.y);
        }
    }
}
exports.scrollHorizontally = scrollHorizontally;
var queue = [];
var queueRunning;
function addToScrollQueue(fn, element, x, y) {
    queue.push({ fn: fn, element: element, x: x, y: y });
    if (!queueRunning) {
        processQueue();
    }
}
function processQueue() {
    if (queue.length > 0) {
        queueRunning = true;
        var _a = queue.shift(), fn = _a.fn, element = _a.element, x = _a.x, y = _a.y;
        fn(element, x, y).then(processQueue).catch(processQueue);
    }
    else {
        queueRunning = false;
    }
}
//# sourceMappingURL=scrolling.js.map